import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Class that can request data via internet protocol.
 * 
 * @author Johannes A. Stork
 *
 */
public class DataDownloader {
	public static final int TIMEOUT_TIME = 300;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
	
		try{
			PrintWriter protocol = new PrintWriter(new File("list.dat"));
			
			for(int j = 0; j < 5; j++) {
				for(int i = 1; i < 1000; i++) {
					//DataDownloader.requestUDP(i * 1000, protocol);
					//protocol.flush();
					DataDownloader.requestTCP(10000000, protocol);
					//protocol.flush();
					//DataDownloader.requestHTTP(protocol);
					protocol.flush();
					//Thread.sleep(1000);
					//System.gc();
				}
			}
			protocol.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Requests 100 MB of data via HTTP 
	 * @param protocol
	 */
	public static void requestHTTP(PrintWriter protocol) {
		System.out.println("HTTP-Request");
		
		try{
			/// Setup URL
			URL url = new URL ("http://vulcano.informatik.uni-freiburg.de/file_100M"); 
			HttpURLConnection httpConnection = (HttpURLConnection) url.openConnection();  
			httpConnection.setReadTimeout(DataDownloader.TIMEOUT_TIME);
			
			/// Perform HTTP connection
			httpConnection.connect();
			long starTime = System.currentTimeMillis();
			long stopTime = System.currentTimeMillis();
			
			InputStreamReader is = new InputStreamReader(httpConnection.getInputStream());

			int i = 0;
			try {
				/// Read the stream
				while(is.ready()) {
					long l = is.skip(1);
					stopTime = System.currentTimeMillis();
					i++;
				}
			} catch (Exception e) {
				System.out.println("Timeout");
			}

			stopTime = System.currentTimeMillis();			
			long time = stopTime - starTime;
			//double errorRate = (double) packetCount / ((double) numberOfBytes / (double) 1000);
			double transfRate = (double) ((double) 100 * 1000 / 1000.0) / ((double) time / 1000.0); /// KB/s
			
			is.close();
			System.out.println(i / 1000 + " KB");
			System.out.println(transfRate + " KB/s " + 0.0 + " err");
			
			protocol.println("HTP " + transfRate + " " + 0.0 + " " + 100000000);
			
			protocol.flush();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
		
		}
	}
	
	/**
	 * Requests data via TCP
	 * @param numberOfBytes
	 * @param protocol
	 */
	public static void requestTCP(int numberOfBytes, PrintWriter protocol) {
		System.out.println("TCP-Request");
		
		try {
			/// Setup connection
			Socket s = new Socket(InetAddress.getByName("vulcano.informatik.uni-freiburg.de"), 9999);
			s.setSoTimeout(DataDownloader.TIMEOUT_TIME);
			
			/// Write to the server
			PrintWriter pw = new PrintWriter(s.getOutputStream());
			pw.print("GET /" + numberOfBytes + " HTTP/1.1\0");
			pw.flush();
			
			long starTime = System.currentTimeMillis();
			long stopTime = System.currentTimeMillis();
			
			InputStreamReader is = new InputStreamReader(s.getInputStream());
			
			int i = 0;
			try {
				/// Read the stream
				while(s.isConnected()) {
					//System.out.println(i);
					is.read();
					stopTime = System.currentTimeMillis();
					i++;
				}
			} catch (Exception e) {
				System.out.println("Timeout");
			}
			
			long time = stopTime - starTime;
			double transfRate = (double) ((double) i / 1000.0) / ((double) time / 1000.0); /// KB/s
			
			is.close();
			pw.close();
			s.close();

			System.out.println(i / 1000 + " KB");
			System.out.println(transfRate + " KB/s " + 0.0 + " err");
			
			protocol.println("TCP " + transfRate + " " + 0.0 + " " + numberOfBytes);
			

		} catch (Exception e) {
			e.printStackTrace();
		} finally {

			protocol.flush();
		}
	}
	
	/**
	 * Requests data via UDP
	 * @param numberOfBytes
	 * @param protocol
	 */
	public static void requestUDP(int numberOfBytes, PrintWriter protocol) {
		System.out.println("UDP-Request " + numberOfBytes);
		
		/// Construct a zero-terminated ASCII-String
		String s = new String(numberOfBytes + "\0");
		byte[] sendData = s.getBytes();
		
		/// Pack datagramm (port and host are taken from the socket)
		DatagramPacket dpSend = new DatagramPacket(sendData, sendData.length);

		/// Receive datagram
		byte[] receiveData = new byte[1000];
		DatagramPacket dpReceive = new DatagramPacket(receiveData, 1000);
		
		try {
			/// Create socket
			DatagramSocket ds = new DatagramSocket(8889);
			/// Perform connection
			ds.connect(InetAddress.getByName("vulcano.informatik.uni-freiburg.de"), 8888);
			/// Set the timeout (do not wait for ever)
			ds.setSoTimeout(DataDownloader.TIMEOUT_TIME);
			
			/// Send the datagram
			ds.send(dpSend);
			long starTime = System.currentTimeMillis();
			long stopTime = System.currentTimeMillis();
			
			boolean timeout = false;
			int packetCount = 0;
			
			while(!timeout) {
				try{
				/// Wait for packet
				ds.receive(dpReceive);
				stopTime = System.currentTimeMillis();
				
				/// Get the data from the packet
				byte[] info = dpReceive.getData();
				
				String reply = new String(Arrays.copyOf(info, 10));
				int number = Integer.parseInt(reply);
			
				packetCount++;
				
				} catch(SocketTimeoutException e) {
					System.out.println("Timeout");
					timeout = true;
				}
			}
			
			ds.disconnect();
			ds.close();
			
			long time = stopTime - starTime;
			double errorRate = 1 - ((double) packetCount * 1000) / ((double) numberOfBytes);
			double transfRate = (double) ((double) packetCount) / ((double) time / 1000.0); /// KB/s
			
			System.out.println(packetCount + " KB");
			System.out.println(transfRate + " KB/s " + errorRate + " err");
			
			protocol.println("UDP " + transfRate + " " + errorRate + " " + numberOfBytes);
			protocol.flush();
		} catch(Exception e) {
			e.printStackTrace();
		} finally {

		}
	}

}
