import java.net.*;
import java.io.*;
/**
 * 
 * A small webserver, that processes only one type of queries (GET).
 * 
 * @author Johannes A. Stork
 *
 */

public class SmallWebserver extends Thread{
	protected int port;
	protected ServerSocket serverSocket;
	protected LineQueryProcessor lqp;
	
	
	/**
	 * Constuctor for this class that selects the port
	 * @param port The port to listen on
	 */
	SmallWebserver(int port, LineQueryProcessor lqp) {
		this.port = port;
		this.lqp = lqp;
	}
	
	/**
	 * Begins listening on selected port
	 *
	 */
	public void run() {
		try {
			/// Create the server socket
			this.serverSocket = new ServerSocket(port);
			
			/// Main loop of the server, that serves one client
			/// after another
			while(true) {
				System.out.println("Waiting on port " + this.port);
				
				/// Wait until a client connects and create a socket
				/// for the connection to this client
				Socket socket = this.serverSocket.accept();
				
				/// Create a reader for the tcp connection
				BufferedReader br = new BufferedReader(
						new InputStreamReader(socket.getInputStream()));
				
				/// Read the first line from the tcp connection
				String firstLine = br.readLine();
				
				/// Get a answer to the query
				String answer = this.lqp.process(firstLine);
				
				/// Create a writer for the tcp connection
				PrintWriter pw = new PrintWriter(socket.getOutputStream());
				
				/// Write header
				pw.println("HTTP/1.1 200 OK");
				/// Write empty line (body start)
				pw.println("");
				
				/// Write answer
				pw.println(answer);
				
				/// Force buffer transmission
				pw.flush();
				
				/// To a clean disconnection
				socket.close();
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
}
